/* Arduino ultrasonic sensor hack for robot mower

---For Tianchen TC-G158 wiring-------------------------------------------------- 

Robot bumpers pinout (P8) is (left-to-right):
1---GND
2---left bumper signal pin  (we use this pin to indicate obstacles)
3---GND
4---right bumper signal pin  (we use this pin to indicate obstacles)

Robot P6 pinout is (top-to-bottom):
1---VCC (+5V)
4---GND
--------------------------------------------------------------------------------

*/

//#include <NewPing.h>

#define TRIGGER_PIN  12  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     11  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define PIEZO_PIN    3
#define LED_PIN      13
#define BUMPER1_PIN   7
#define BUMPER2_PIN   8
#define MAX_DISTANCE 200 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.
#define NO_ECHO 0
#define MAX_ECHO_TIME 5000
#define PING_MEDIAN_DELAY 29


enum STATE
{
  BUMPER1,
  BUMPER2,
  OFF
};

//NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

void setup() {
  pinMode(BUMPER1_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
  pinMode(BUMPER2_PIN, INPUT);   // set bumper pin to tri-state (high impedance)  
  digitalWrite(BUMPER1_PIN, LOW); // disable bumper pin pull-up
  digitalWrite(BUMPER2_PIN, LOW); // disable bumper pin pull-up
  pinMode(3, OUTPUT);
  pinMode(TRIGGER_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
  pinMode(LED_PIN, OUTPUT);  
  Serial.begin(115200); // Open serial monitor at 115200 baud to see ping results.  
}

unsigned int ping(){
  unsigned int uS;  
  digitalWrite(TRIGGER_PIN, LOW); 
  delayMicroseconds(2); 
  digitalWrite(TRIGGER_PIN, HIGH);
  delayMicroseconds(10); 
  digitalWrite(TRIGGER_PIN, LOW);
  uS = pulseIn(ECHO_PIN, HIGH, MAX_ECHO_TIME + 1000);
  if (uS > MAX_ECHO_TIME) uS = NO_ECHO;
  return uS;
}


unsigned int ping_median(uint8_t it) {
  unsigned int uS[it], last;
  uint8_t j, i = 0;
  uS[0] = NO_ECHO;
  while (i < it) {
    last = ping();           // Send ping.
    if (last == NO_ECHO) {   // Ping out of range.
	it--;                // Skip, don't include as part of median.
	last = MAX_ECHO_TIME; // Adjust "last" variable so delay is correct length.
    } else {                       // Ping in range, include as part of median.
	if (i > 0) {               // Don't start sort till second ping.
		for (j = i; j > 0 && uS[j - 1] < last; j--) // Insertion sort loop.
		  uS[j] = uS[j - 1]; // Shift ping array to correct position for sort insertion.
	 } else j = 0;              // First ping is starting point for sort.
	 uS[j] = last;              // Add last ping to array in sorted position.
         i++;                       // Move to next ping.
     }
     if (i < it) delay(PING_MEDIAN_DELAY - (last >> 10)); // Millisecond delay between pings.
   }
   return (uS[it >> 1]); // Return the ping distance median.
}


static int oldfreq = 0;
static unsigned long lastBumperTime = 0;
static enum STATE state = OFF;
static int  bumper1Counter = 0;
static int  bumper2Counter = 0;
static unsigned long nextInfo = millis() + 1000;
static long dist = 0;     
  
  
void loop() {
  long uS; 
  long freq = 0;
  
  delay(50);   
  uS = ping_median(5);     
  dist = uS/57;
  // (Gleitender Durchschnitt mit Integer)
  // M1 = M * (1-w) + A * w    => (w = 2^-n) => M1 = M * (1 – 2^-n) + A * 2^-n
  // n=3 : M1 = (M * 8 – M + A) / 8
  //long newDist = uS/57;
  //dist = (dist * 8 - dist + newDist) / 8;
  
  if ((millis() > lastBumperTime + 2000)){
    // bumper trigger timeout expired
    if ((dist > 0) && (dist < 30)){    
      // we want to trigger this bumper order: 1, 1, 2, 1, 1, 2, 1, 1, 2, etc.
      digitalWrite(LED_PIN, HIGH);      
      if (bumper1Counter < 2) {
        state = BUMPER1;        
        bumper1Counter++;                
      } else {        
        state = BUMPER2;
        bumper2Counter++;                
        if (bumper2Counter == 1) {
          bumper1Counter = bumper2Counter = 0;
        }
      }
      
      if (state == BUMPER1){
        pinMode(BUMPER1_PIN, OUTPUT);   // change bumper from tri-state to low
      } else if (state == BUMPER2){
        pinMode(BUMPER2_PIN, OUTPUT);   // change bumper from tri-state to low
      }            
      lastBumperTime = millis();      
    }    
  } 
  if ((millis() > lastBumperTime + 1000)) {
    state = OFF;
    digitalWrite(LED_PIN, LOW);
    pinMode(BUMPER1_PIN, INPUT);   // change bumper from low to tri-state 
    pinMode(BUMPER2_PIN, INPUT);   // change bumper from low to tri-state 
  }
  
  if ((dist > 0) || (dist < 80)){
    freq = dist * 50;
  } else freq = 0;
  if (freq != oldfreq){
    if (freq > 0) {
      //digitalWrite(LED_PIN, HIGH);
      tone(PIEZO_PIN, freq);          
    } else {
      //digitalWrite(LED_PIN, LOW);
      noTone(PIEZO_PIN);
    }
    oldfreq = freq;
  }

  if (millis() > nextInfo){    
    nextInfo = millis() + 1000;
    Serial.print("uS ");
    Serial.print(uS); 
    Serial.print(" dist ");
    Serial.print(dist); 
    Serial.print(" freq ");
    Serial.print(freq); 
    Serial.print(" state ");
    switch (state)
    {
       case BUMPER1:  Serial.print("B1"); break;
       case BUMPER2:  Serial.print("B2"); break;
       case OFF:  Serial.print("OFF"); break;       
    }
    Serial.println(""); 
  }
  
      
  /*if ((dist > 0) && (dist < 40))
    analogWrite(PIEZO_PIN, 127);
  else if (dist >= 40) 
    analogWrite(PIEZO_PIN, 253);  
  else analogWrite(PIEZO_PIN, 255);*/
  
}


